package tsdb;

import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.StringUtils;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

class Util {
    public static void ensureDir(String dir) {
        java.io.File f = new java.io.File(dir);
        if (f.exists()) {
            if (!f.isDirectory()) {
                throw new RuntimeException("File " + dir + " exists and is not a directory");
            }
        } else {
            if (!f.mkdirs()) {
                throw new RuntimeException("Could not create directory " + dir);
            }
        }
    }

    public static String escapeIntoSingleQuotedString(String s) {
        StringBuilder sb = new StringBuilder();
        sb.append('\'');
        for (int i = 0; i < s.length(); i++) {
            char c = s.charAt(i);
            if (c == '\'' || c == '\\') {
                sb.append('\\');
                sb.append(c);
            } else {
                sb.append(c);
            }
        }
        sb.append('\'');
        return sb.toString();
    }

    public static String pathCombine(String path1, String path2) {
        return Paths.get(path1, path2).toString();
    }

    public static void renameFileToExtension(String path, String extension) {
        java.io.File f = new java.io.File(path);
        String oldExt = FilenameUtils.getExtension(path);
        if (!StringUtils.isEmpty(oldExt)) {
            path = FilenameUtils.removeExtension(path);
        }
        if (extension == null) {
            extension = "";
        }
        if (!extension.isEmpty() && !StringUtils.startsWith(extension, ".")) {
            extension = "." + extension;
        }
        String newPath = path + extension;
//        if (!f.exists()) {
//            throw new RuntimeException("File " + path + " does not exist");
//        }
//        if (!f.isFile()) {
//            throw new RuntimeException("File " + path + " is not a file");
//        }
        java.io.File newFile = new java.io.File(newPath);
        if (f.renameTo(newFile)) {
            return;
        }
        // Failed, try to delete the file first
        if (!newFile.delete()) {
            throw new RuntimeException("Could not delete file `" + newPath + "`");
        }
        // Try again
        if (!f.renameTo(newFile)) {
            throw new RuntimeException("Could not rename file `" + path + "` to `" + newPath + "`");
        }
    }

    public static void writeFileExact(FileChannel fc, ByteBuffer buf) {
        try {
            while (buf.hasRemaining()) {
                if (fc.write(buf) == 0) {
                    throw new RuntimeException("File write returned 0; no space on device?");
                }
            }
        } catch (IOException e) {
//            rethrow(e);
            throw new RuntimeException(e);
        }
    }

    /**
     * Cast a CheckedException as an unchecked one.
     *
     * @param throwable to cast
     * @param <T>       the type of the Throwable
     * @return this method will never return a Throwable instance, it will just throw it.
     * @throws T the throwable as an unchecked throwable
     */
    @SuppressWarnings("unchecked")
    public static <T extends Throwable> void rethrow(Throwable throwable) throws T {
        throw (T) throwable; // rely on vacuous cast
    }

    public static String getFormattedTimestamp(LocalDateTime t) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd_HH-mm-ss.SSSSSS");
        return t.format(formatter);
    }
}
